/**
 * Copyright © 2014-2023 PDF Technologies, Inc. All Rights Reserved.
 * <p>
 * THIS SOURCE CODE AND ANY ACCOMPANYING DOCUMENTATION ARE PROTECTED BY INTERNATIONAL COPYRIGHT LAW
 * AND MAY NOT BE RESOLD OR REDISTRIBUTED. USAGE IS BOUND TO THE ComPDFKit LICENSE AGREEMENT.
 * UNAUTHORIZED REPRODUCTION OR DISTRIBUTION IS SUBJECT TO CIVIL AND CRIMINAL PENALTIES.
 * This notice may not be removed from this file.
 */

package com.compdfkit.tools.annotation.pdfproperties.pdfstyle;


import static com.compdfkit.tools.common.views.pdfstyle.CStyleType.ANNOT_ARROW;
import static com.compdfkit.tools.common.views.pdfstyle.CStyleType.ANNOT_LINE;

import android.text.TextUtils;

import com.compdfkit.core.annotation.CPDFFreetextAnnotation;
import com.compdfkit.core.annotation.CPDFLineAnnotation;
import com.compdfkit.core.annotation.CPDFTextAlignment;
import com.compdfkit.core.annotation.CPDFTextAttribute;
import com.compdfkit.tools.common.views.pdfstyle.CAnnotStyle;
import com.compdfkit.tools.common.views.pdfstyle.CBasicOnStyleChangeListener;
import com.compdfkit.tools.common.views.pdfstyle.CStyleType;
import com.compdfkit.tools.common.views.pdfstyle.manager.CStyleProvider;
import com.compdfkit.tools.common.views.pdfview.CPDFViewCtrl;
import com.compdfkit.ui.attribute.CPDFAnnotAttribute;
import com.compdfkit.ui.attribute.CPDFAttributeDataFether;
import com.compdfkit.ui.attribute.CPDFCircleAttr;
import com.compdfkit.ui.attribute.CPDFFreetextAttr;
import com.compdfkit.ui.attribute.CPDFHighlightAttr;
import com.compdfkit.ui.attribute.CPDFInkAttr;
import com.compdfkit.ui.attribute.CPDFLineAttr;
import com.compdfkit.ui.attribute.CPDFSquareAttr;
import com.compdfkit.ui.attribute.CPDFSquigglyAttr;
import com.compdfkit.ui.attribute.CPDFStampAttr;
import com.compdfkit.ui.attribute.CPDFStrikeoutAttr;
import com.compdfkit.ui.attribute.CPDFTextAttr;
import com.compdfkit.ui.attribute.CPDFUnderlineAttr;
import com.compdfkit.ui.attribute.IAttributeCallback;
import com.compdfkit.ui.attribute.form.CPDFCheckboxAttr;
import com.compdfkit.ui.attribute.form.CPDFComboboxAttr;
import com.compdfkit.ui.attribute.form.CPDFListboxAttr;
import com.compdfkit.ui.attribute.form.CPDFPushButtonAttr;
import com.compdfkit.ui.attribute.form.CPDFRadiobuttonAttr;
import com.compdfkit.ui.attribute.form.CPDFTextfieldAttr;
import com.compdfkit.ui.reader.CPDFReaderView;

import java.util.LinkedHashSet;

public class CPDFGlobalAnnotStyle extends CBasicOnStyleChangeListener implements CStyleProvider {

    private CPDFReaderView readerView;

    private boolean onStore;

    public CPDFGlobalAnnotStyle(CPDFReaderView readerView, boolean onStore) {
        this.readerView = readerView;
        this.onStore = onStore;
    }

    public CPDFGlobalAnnotStyle(CPDFReaderView readerView) {
        this.readerView = readerView;
        this.onStore = false;
    }

    public CPDFGlobalAnnotStyle(CPDFViewCtrl pdfView) {
        this.readerView = pdfView.getCPdfReaderView();
        this.onStore = false;
    }

    public CPDFGlobalAnnotStyle(CPDFViewCtrl pdfView, boolean onStore) {
        this.readerView = pdfView.getCPdfReaderView();
        this.onStore = onStore;
    }

    @Override
    public void updateStyle(CAnnotStyle annotStyle) {
        LinkedHashSet<CAnnotStyle> linkedHashSet = new LinkedHashSet<>();
        linkedHashSet.add(annotStyle);
        updateStyle(linkedHashSet);
    }

    @Override
    public void updateStyle(LinkedHashSet<CAnnotStyle> annotStyle) {
        CPDFAnnotAttribute attribute = readerView.getReaderAttribute().getAnnotAttribute();
        for (CAnnotStyle params : annotStyle) {
            if (params == null) {
                continue;
            }
            IAttributeCallback callback = null;
            switch (params.getType()) {
                case ANNOT_TEXT:
                    CPDFTextAttr textAttr = attribute.getTextAttr();
                    textAttr.setColor(params.getColor());
                    textAttr.setAlpha(params.getOpacity());
                    callback = textAttr;
                    break;
                case ANNOT_HIGHLIGHT:
                    CPDFHighlightAttr highlightAttr = attribute.getHighlightAttr();
                    highlightAttr.setColor(params.getColor());
                    highlightAttr.setAlpha(params.getOpacity());
                    callback = highlightAttr;
                    break;
                case ANNOT_UNDERLINE:
                    CPDFUnderlineAttr underlineAttr = attribute.getUnderlineAttr();
                    underlineAttr.setColor(params.getColor());
                    underlineAttr.setAlpha(params.getOpacity());
                    callback = underlineAttr;
                    break;
                case ANNOT_SQUIGGLY:
                    CPDFSquigglyAttr squigglyAttr = attribute.getSquigglyAttr();
                    squigglyAttr.setColor(params.getColor());
                    squigglyAttr.setAlpha(params.getOpacity());
                    callback = squigglyAttr;
                    break;
                case ANNOT_STRIKEOUT:
                    CPDFStrikeoutAttr strikeoutAttr = attribute.getStrikeoutAttr();
                    strikeoutAttr.setColor(params.getColor());
                    strikeoutAttr.setAlpha(params.getOpacity());
                    callback = strikeoutAttr;
                    break;
                case ANNOT_INK:
                    CPDFInkAttr inkAttr = attribute.getInkAttr();
                    inkAttr.setColor(params.getColor());
                    inkAttr.setAlpha(params.getOpacity());
                    inkAttr.setBorderWidth(params.getBorderWidth());
                    inkAttr.setEraseWidth(params.getEraserWidth());
                    if (!onStore) {
                        readerView.invalidateAllChildren();
                    }
                    break;
                case ANNOT_SQUARE:
                    CPDFSquareAttr squareAttr = attribute.getSquareAttr();
                    squareAttr.setFillColor(params.getFillColor());
                    squareAttr.setFillAlpha(params.getFillColorOpacity());
                    squareAttr.setBorderColor(params.getLineColor());
                    squareAttr.setBorderAlpha(params.getLineColorOpacity());
                    squareAttr.setBorderWidth(params.getBorderWidth());
                    squareAttr.setBorderStyle(params.getBorderStyle());
                    break;
                case ANNOT_CIRCLE:
                    CPDFCircleAttr circleAttr = attribute.getCircleAttr();
                    circleAttr.setFillColor(params.getFillColor());
                    circleAttr.setFillAlpha(params.getFillColorOpacity());
                    circleAttr.setBorderColor(params.getLineColor());
                    circleAttr.setBorderAlpha(params.getLineColorOpacity());
                    circleAttr.setBorderWidth(params.getBorderWidth());
                    circleAttr.setBorderStyle(params.getBorderStyle());
                    break;
                case ANNOT_LINE:
                case ANNOT_ARROW:
                    CPDFLineAttr lineAttr = attribute.getLineAttr();
                    lineAttr.setBorderColor(params.getLineColor());
                    lineAttr.setBorderAlpha(params.getLineColorOpacity());
                    lineAttr.setFillColor(params.getFillColor());
                    lineAttr.setFillAlpha(params.getFillColorOpacity());
                    lineAttr.setBorderWidth(params.getBorderWidth());
                    lineAttr.setBorderStyle(params.getBorderStyle());
                    lineAttr.setLineType(params.getStartLineType(), params.getTailLineType());
                    if (params.getType() == ANNOT_ARROW) {
                        CPDFAttributeDataFether dataFether = new CPDFAttributeDataFether(readerView.getContext());
                        dataFether.setIntValue("custom_line_type", "custom_head_type", params.getStartLineType().id);
                        dataFether.setIntValue("custom_line_type", "custom_tail_type", params.getTailLineType().id);
                    }
                    break;
                case ANNOT_FREETEXT:
                    CPDFFreetextAttr freetextAttr = attribute.getFreetextAttr();
                    switch (params.getAlignment()) {
                        case LEFT:
                            freetextAttr.setAlignment(CPDFFreetextAnnotation.Alignment.ALIGNMENT_LEFT);
                            break;
                        case CENTER:
                            freetextAttr.setAlignment(CPDFFreetextAnnotation.Alignment.ALIGNMENT_CENTER);
                            break;
                        case RIGHT:
                            freetextAttr.setAlignment(CPDFFreetextAnnotation.Alignment.ALIGNMENT_RIGHT);
                            break;
                        default:
                            freetextAttr.setAlignment(CPDFFreetextAnnotation.Alignment.ALIGNMENT_UNKNOW);
                            break;
                    }
                    freetextAttr.setAlpha(params.getTextColorOpacity());
                    String fontName = CPDFTextAttribute.FontNameHelper.obtainFontName(params.getFontType(), params.isFontBold(), params.isFontItalic());
                    freetextAttr.setTextAttribute(new CPDFTextAttribute(fontName, params.getFontSize(), params.getTextColor()));
                    break;
                case ANNOT_SIGNATURE:
                    CPDFStampAttr stampAttr = attribute.getStampAttr();
                    stampAttr.setImagePath(params.getImagePath(), true);
                    break;
                case ANNOT_STAMP:
                case ANNOT_PIC:
                    CPDFStampAttr stampAttr1 = attribute.getStampAttr();
                    if (params.getStandardStamp() != null) {
                        stampAttr1.setStandardStamp(params.getStandardStamp());
                    } else if (params.getTextStamp() != null) {
                        stampAttr1.setTextStamp(params.getTextStamp());
                    } else if (!TextUtils.isEmpty(params.getImagePath())) {
                        stampAttr1.setImagePath(params.getImagePath(), false);
                    }
                    break;
                case FORM_TEXT_FIELD:
                    CPDFTextfieldAttr textFieldAttr = attribute.getTextfieldAttr();
                    textFieldAttr.setBorderWidth(params.getBorderWidth());
                    textFieldAttr.setBorderColor(params.getLineColor());
                    textFieldAttr.setFillColor(params.getFillColor());
                    String textFieldFontName = CPDFTextAttribute.FontNameHelper.obtainFontName(params.getFontType(), params.isFontBold(), params.isFontItalic());
                    textFieldAttr.setTextAttribute(new CPDFTextAttribute(textFieldFontName, params.getFontSize(), params.getTextColor()));
                    textFieldAttr.setMultiline(params.isFormMultiLine());
                    switch (params.getAlignment()) {
                        case LEFT:
                            textFieldAttr.setTextAlignment(CPDFTextAlignment.ALIGNMENT_LEFT);
                            break;
                        case CENTER:
                            textFieldAttr.setTextAlignment(CPDFTextAlignment.ALIGNMENT_CENTER);
                            break;
                        case RIGHT:
                            textFieldAttr.setTextAlignment(CPDFTextAlignment.ALIGNMENT_RIGHT);
                            break;
                        default:
                            textFieldAttr.setTextAlignment(CPDFTextAlignment.ALIGNMENT_UNKNOWN);
                            break;
                    }
                    break;
                case FORM_CHECK_BOX:
                    CPDFCheckboxAttr checkboxAttr = attribute.getCheckboxAttr();
                    checkboxAttr.setCheckStyle(params.getCheckStyle());
                    checkboxAttr.setBorderWidth(params.getBorderWidth());
                    checkboxAttr.setColor(params.getColor());
                    checkboxAttr.setBorderColor(params.getLineColor());
                    checkboxAttr.setFillColor(params.getFillColor());
                    checkboxAttr.setChecked(params.isChecked());
                    break;
                case FORM_RADIO_BUTTON:
                    CPDFRadiobuttonAttr radiobuttonAttr = attribute.getRadiobuttonAttr();
                    radiobuttonAttr.setCheckStyle(params.getCheckStyle());
                    radiobuttonAttr.setBorderWidth(params.getBorderWidth());
                    radiobuttonAttr.setColor(params.getColor());
                    radiobuttonAttr.setBorderColor(params.getLineColor());
                    radiobuttonAttr.setFillColor(params.getFillColor());
                    radiobuttonAttr.setChecked(params.isChecked());
                    break;
                case FORM_LIST_BOX:
                    CPDFListboxAttr listBoxAttr = attribute.getListboxAttr();
                    listBoxAttr.setFontSize(params.getFontSize());
                    listBoxAttr.setFontColor(params.getTextColor());
                    listBoxAttr.setFillColor(params.getFillColor());
                    listBoxAttr.setBorderWidth(params.getBorderWidth());
                    listBoxAttr.setBorderColor(params.getLineColor());
                    break;
                case FORM_COMBO_BOX:
                    CPDFComboboxAttr comboBoxAttr = attribute.getComboboxAttr();
                    comboBoxAttr.setFillColor(params.getFillColor());
                    comboBoxAttr.setFontSize(params.getFontSize());
                    comboBoxAttr.setFontColor(params.getTextColor());
                    comboBoxAttr.setBorderWidth(params.getBorderWidth());
                    comboBoxAttr.setBorderColor(params.getLineColor());
                    break;
                case FORM_PUSH_BUTTON:
                    CPDFPushButtonAttr pushButtonAttr = attribute.getPushButtonAttr();
                    pushButtonAttr.setFillColor(params.getFillColor());
                    pushButtonAttr.setBorderColor(params.getLineColor());
                    pushButtonAttr.setBorderWidth(params.getBorderWidth());
                    pushButtonAttr.setFontColor(params.getTextColor());
                    pushButtonAttr.setFontSize(params.getFontSize());
                    pushButtonAttr.setButtonTitle(params.getFormDefaultValue());
                    break;
                default:
                    break;
            }
            if (onStore) {
                if (callback != null) {
                    callback.onstore();
                }
            }
        }
    }

    @Override
    public CAnnotStyle getStyle(CStyleType type) {
        CAnnotStyle style = new CAnnotStyle(type);
        CPDFAnnotAttribute attribute = readerView.getReaderAttribute().getAnnotAttribute();
        switch (type) {
            case ANNOT_TEXT:
                CPDFTextAttr textAttr = attribute.getTextAttr();
                style.setColor(textAttr.getColor());
                style.setOpacity(textAttr.getAlpha());
                break;
            case ANNOT_HIGHLIGHT:
                CPDFHighlightAttr highlightAttr = attribute.getHighlightAttr();
                style.setColor(highlightAttr.getColor());
                style.setOpacity(highlightAttr.getAlpha());
                break;
            case ANNOT_UNDERLINE:
                CPDFUnderlineAttr underlineAttr = attribute.getUnderlineAttr();
                style.setColor(underlineAttr.getColor());
                style.setOpacity(underlineAttr.getAlpha());
                break;
            case ANNOT_SQUIGGLY:
                CPDFSquigglyAttr squigglyAttr = attribute.getSquigglyAttr();
                style.setColor(squigglyAttr.getColor());
                style.setOpacity(squigglyAttr.getAlpha());
                break;
            case ANNOT_STRIKEOUT:
                CPDFStrikeoutAttr strikeoutAttr = attribute.getStrikeoutAttr();
                style.setColor(strikeoutAttr.getColor());
                style.setOpacity(strikeoutAttr.getAlpha());
                break;
            case ANNOT_INK:
                CPDFInkAttr inkAttr = attribute.getInkAttr();
                style.setColor(inkAttr.getColor());
                style.setOpacity(inkAttr.getAlpha());
                style.setBorderWidth(inkAttr.getBorderWidth());
                break;
            case ANNOT_SQUARE:
                CPDFSquareAttr squareAttr = attribute.getSquareAttr();
                style.setFillColor(squareAttr.getFillColor());
                style.setFillColorOpacity(squareAttr.getFillAlpha());
                style.setBorderColor(squareAttr.getBorderColor());
                style.setLineColorOpacity(squareAttr.getBorderAlpha());
                style.setBorderWidth(squareAttr.getBorderWidth());
                style.setBorderStyle(squareAttr.getBorderStyle());
                break;
            case ANNOT_CIRCLE:
                CPDFCircleAttr circleAttr = attribute.getCircleAttr();
                style.setFillColor(circleAttr.getFillColor());
                style.setFillColorOpacity(circleAttr.getFillAlpha());
                style.setBorderColor(circleAttr.getBorderColor());
                style.setLineColorOpacity(circleAttr.getBorderAlpha());
                style.setBorderWidth(circleAttr.getBorderWidth());
                style.setBorderStyle(circleAttr.getBorderStyle());
                break;
            case ANNOT_LINE:
            case ANNOT_ARROW:
                CPDFLineAttr lineAttr = attribute.getLineAttr();
                style.setBorderColor(lineAttr.getBorderColor());
                style.setLineColorOpacity(lineAttr.getBorderAlpha());
                style.setFillColor(lineAttr.getFillColor());
                style.setFillColorOpacity(lineAttr.getFillAlpha());
                style.setBorderStyle(lineAttr.getBorderStyle());
                style.setBorderWidth(lineAttr.getBorderWidth());
                if (type == ANNOT_LINE) {
                    style.setStartLineType(CPDFLineAnnotation.LineType.LINETYPE_NONE);
                    style.setTailLineType(CPDFLineAnnotation.LineType.LINETYPE_NONE);
                } else {
                    CPDFAttributeDataFether dataFether = new CPDFAttributeDataFether(readerView.getContext());
                    CPDFLineAnnotation.LineType headType = CPDFLineAnnotation.LineType.valueOf(dataFether.getIntValue("custom_line_type", "custom_head_type", CPDFLineAnnotation.LineType.LINETYPE_NONE.id));
                    CPDFLineAnnotation.LineType tailType = CPDFLineAnnotation.LineType.valueOf(dataFether.getIntValue("custom_line_type", "custom_tail_type", CPDFLineAnnotation.LineType.LINETYPE_ARROW.id));
                    style.setStartLineType(headType);
                    style.setTailLineType(tailType);
                }

                break;
            case ANNOT_FREETEXT:
                CPDFFreetextAttr freetextAttr = attribute.getFreetextAttr();
                switch (freetextAttr.getAlignment()) {
                    case ALIGNMENT_LEFT:
                        style.setAlignment(CAnnotStyle.Alignment.LEFT);
                        break;
                    case ALIGNMENT_CENTER:
                        style.setAlignment(CAnnotStyle.Alignment.CENTER);
                        break;
                    case ALIGNMENT_RIGHT:
                        style.setAlignment(CAnnotStyle.Alignment.RIGHT);
                        break;
                    default:
                        style.setAlignment(CAnnotStyle.Alignment.UNKNOWN);
                        break;
                }
                style.setTextColorOpacity(freetextAttr.getAlpha());
                CPDFTextAttribute textAttribute = freetextAttr.getTextAttribute();
                style.setFontColor(textAttribute.getColor());
                style.setFontType(CPDFTextAttribute.FontNameHelper.getFontType(textAttribute.getFontName()));
                style.setFontBold(CPDFTextAttribute.FontNameHelper.isBold(textAttribute.getFontName()));
                style.setFontItalic(CPDFTextAttribute.FontNameHelper.isItalic(textAttribute.getFontName()));
                style.setFontSize((int) textAttribute.getFontSize());
                break;
            case FORM_TEXT_FIELD:
                CPDFTextfieldAttr textFieldAttr = attribute.getTextfieldAttr();
                //text fields
                style.setHideForm(false);
                style.setFormMultiLine(textFieldAttr.isMultiline());
                style.setFontColor(textFieldAttr.getFontColor());
                style.setBorderColor(textFieldAttr.getBorderColor());
                style.setFillColor(textFieldAttr.getFillColor());
                CPDFTextAttribute.FontNameHelper.FontType fontType = CPDFTextAttribute.FontNameHelper.getFontType(textFieldAttr.getFontName());
                style.setFontType(fontType);
                style.setFontBold(CPDFTextAttribute.FontNameHelper.isBold(textFieldAttr.getFontName()));
                style.setFontItalic(CPDFTextAttribute.FontNameHelper.isItalic(textFieldAttr.getFontName()));
                style.setFontSize((int) textFieldAttr.getFontSize());
                style.setFormDefaultValue("");
                switch (textFieldAttr.getTextAlignment()) {
                    case ALIGNMENT_LEFT:
                        style.setAlignment(CAnnotStyle.Alignment.LEFT);
                        break;
                    case ALIGNMENT_CENTER:
                        style.setAlignment(CAnnotStyle.Alignment.CENTER);
                        break;
                    case ALIGNMENT_RIGHT:
                        style.setAlignment(CAnnotStyle.Alignment.RIGHT);
                        break;
                    default:
                        style.setAlignment(CAnnotStyle.Alignment.UNKNOWN);
                        break;
                }
                break;
            case FORM_LIST_BOX:
                CPDFListboxAttr listboxAttr = attribute.getListboxAttr();
                style.setFontSize((int) listboxAttr.getFontSize());
                style.setFontColor(listboxAttr.getFontColor());
                style.setFillColor(listboxAttr.getFillColor());
                style.setBorderColor(listboxAttr.getBorderColor());
                style.setBorderWidth(listboxAttr.getBorderWidth());
                CPDFTextAttribute.FontNameHelper.FontType listBoxFontType = CPDFTextAttribute.FontNameHelper.getFontType(listboxAttr.getFontName());
                style.setFontType(listBoxFontType);
                style.setFontBold(CPDFTextAttribute.FontNameHelper.isBold(listboxAttr.getFontName()));
                style.setFontItalic(CPDFTextAttribute.FontNameHelper.isItalic(listboxAttr.getFontName()));
                break;
            case FORM_COMBO_BOX:
                CPDFComboboxAttr comboBoxAttr = attribute.getComboboxAttr();
                style.setFontSize((int) comboBoxAttr.getFontSize());
                style.setFontColor(comboBoxAttr.getFontColor());
                style.setFillColor(comboBoxAttr.getFillColor());
                style.setBorderColor(comboBoxAttr.getBorderColor());
                style.setBorderWidth(comboBoxAttr.getBorderWidth());
                CPDFTextAttribute.FontNameHelper.FontType comBoBoxFontType = CPDFTextAttribute.FontNameHelper.getFontType(comboBoxAttr.getFontName());
                style.setFontType(comBoBoxFontType);
                style.setFontBold(CPDFTextAttribute.FontNameHelper.isBold(comboBoxAttr.getFontName()));
                style.setFontItalic(CPDFTextAttribute.FontNameHelper.isItalic(comboBoxAttr.getFontName()));
                break;
            case FORM_PUSH_BUTTON:
                CPDFPushButtonAttr pushButtonAttr = attribute.getPushButtonAttr();
                //text fields
                style.setHideForm(false);
                style.setFontColor(pushButtonAttr.getFontColor());
                style.setBorderColor(pushButtonAttr.getBorderColor());
                style.setFillColor(pushButtonAttr.getFillColor());
                CPDFTextAttribute.FontNameHelper.FontType pushButtonFontType = CPDFTextAttribute.FontNameHelper.getFontType(pushButtonAttr.getFontName());
                style.setFontType(pushButtonFontType);
                style.setFontBold(CPDFTextAttribute.FontNameHelper.isBold(pushButtonAttr.getFontName()));
                style.setFontItalic(CPDFTextAttribute.FontNameHelper.isItalic(pushButtonAttr.getFontName()));
                style.setFontSize((int) pushButtonAttr.getFontSize());
                style.setFormDefaultValue(pushButtonAttr.getButtonTitle());
                break;
            default:
                break;
        }
        return style;
    }
}
