/**
 * Copyright © 2014-2023 PDF Technologies, Inc. All Rights Reserved.
 * <p>
 * THIS SOURCE CODE AND ANY ACCOMPANYING DOCUMENTATION ARE PROTECTED BY INTERNATIONAL COPYRIGHT LAW
 * AND MAY NOT BE RESOLD OR REDISTRIBUTED. USAGE IS BOUND TO THE ComPDFKit LICENSE AGREEMENT.
 * UNAUTHORIZED REPRODUCTION OR DISTRIBUTION IS SUBJECT TO CIVIL AND CRIMINAL PENALTIES.
 * This notice may not be removed from this file.
 */

package com.compdfkit.tools.common.views.pdfannotationbar;


import android.content.Context;
import android.os.Handler;
import android.os.Looper;
import android.text.TextUtils;
import android.util.AttributeSet;
import android.widget.FrameLayout;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.widget.AppCompatImageView;
import androidx.fragment.app.FragmentManager;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.compdfkit.core.annotation.CPDFAnnotation;
import com.compdfkit.core.undo.CPDFUndoManager;
import com.compdfkit.core.undo.exception.CPDFRedoFailedException;
import com.compdfkit.core.undo.exception.CPDFUndoFailedException;
import com.compdfkit.tools.R;
import com.compdfkit.tools.common.interfaces.COnAnnotationChangeListener;
import com.compdfkit.tools.common.utils.viewutils.CViewUtils;
import com.compdfkit.tools.common.views.pdfannotationbar.adapter.CPDFAnnotationToolListAdapter;
import com.compdfkit.tools.common.views.pdfannotationbar.bean.CAnnotToolBean;
import com.compdfkit.tools.common.views.pdfannotationbar.data.CAnnotationToolDatas;
import com.compdfkit.tools.common.views.pdfproperties.CAnnotationType;
import com.compdfkit.tools.common.views.pdfstyle.CAnnotStyle;
import com.compdfkit.tools.common.views.pdfstyle.CBasicOnStyleChangeListener;
import com.compdfkit.tools.common.views.pdfstyle.CStyleDialogFragment;
import com.compdfkit.tools.common.views.pdfstyle.CStyleType;
import com.compdfkit.tools.common.views.pdfstyle.CStyleUIParams;
import com.compdfkit.tools.common.views.pdfstyle.manager.CStyleManager;
import com.compdfkit.tools.common.views.pdfview.CPDFViewCtrl;
import com.compdfkit.ui.proxy.attach.IInkDrawCallback;

public class CAnnotationToolbar extends FrameLayout {

    private RecyclerView rvAnnotationList;

    private AppCompatImageView ivSetting;

    private AppCompatImageView ivUndo;

    private AppCompatImageView ivRedo;

    private CPDFAnnotationToolListAdapter toolListAdapter;

    private CPDFViewCtrl pdfView;

    private FragmentManager fragmentManager;

    private COnAnnotationChangeListener annotationChangeListener;

    public CAnnotationToolbar(@NonNull Context context) {
        this(context, null);
    }

    public CAnnotationToolbar(@NonNull Context context, @Nullable AttributeSet attrs) {
        this(context, attrs, 0);
    }

    public CAnnotationToolbar(@NonNull Context context, @Nullable AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        initView(context);
    }

    private void initView(Context context) {
        inflate(context, R.layout.tools_annot_tool_bar, this);
        rvAnnotationList = findViewById(R.id.rv_annotation);
        ivSetting = findViewById(R.id.iv_annotation_attr_settings);
        ivUndo = findViewById(R.id.iv_annotation_attr_undo);
        ivRedo = findViewById(R.id.iv_annotation_attr_redo);
        toolListAdapter = new CPDFAnnotationToolListAdapter();
        rvAnnotationList.setLayoutManager(new LinearLayoutManager(getContext(), LinearLayoutManager.HORIZONTAL, false));
        rvAnnotationList.setAdapter(toolListAdapter);
        initListener();
    }

    private void initListener() {
        ivSetting.setEnabled(false);
        ivSetting.setOnClickListener(v -> {
            showAnnotStyleDialog();
        });

        ivUndo.setOnClickListener(v -> {
            undo();
        });
        ivRedo.setOnClickListener(v -> {
            redo();
        });
        toolListAdapter.setOnItemClickListener((adapter, view, position) -> {
            CAnnotToolBean bean = adapter.list.get(position);
            toolListAdapter.selectItem(position);
            switchAnnotationType(bean);
        });
    }

    public void initWithPDFView(CPDFViewCtrl pdfView) {
        this.pdfView = pdfView;
        toolListAdapter.setList(CAnnotationToolDatas.getAnnotationList(pdfView));
        this.pdfView.addOnPDFFocusedTypeChangeListener(type -> {
            if (type == CPDFAnnotation.Type.UNKNOWN) {
                if (toolListAdapter.hasSelectAnnotType()) {
                    toolListAdapter.selectByType(CAnnotationType.UNKNOWN);
                    ivSetting.setEnabled(toolListAdapter.annotEnableSetting());
                }
            }
        });
        redoUndoManager();
    }

    public void setFragmentManager(FragmentManager fragmentManager) {
        this.fragmentManager = fragmentManager;
    }

    private void showAnnotStyleDialog() {
        CViewUtils.hideKeyboard(this);
        CStyleManager styleManager = new CStyleManager(pdfView);
        CStyleType styleType = toolListAdapter.getCurrentAnnotType().getStyleType();
        CAnnotStyle style = styleManager.getStyle(styleType);
        CStyleUIParams styleUiParams = CStyleUIParams.defaultStyle(styleType);
        CStyleDialogFragment dialogFragment = CStyleDialogFragment.newInstance(style);
        dialogFragment.setStyleUiConfig(styleUiParams);
        styleManager.setAnnotStyleFragmentListener(dialogFragment);
        dialogFragment.addAnnotStyleChangeListener(new CBasicOnStyleChangeListener() {
            @Override
            public void onChangeColor(int color) {
                super.onChangeColor(color);
                toolListAdapter.updateItemColor(toolListAdapter.getCurrentAnnotType(), color);
            }

            @Override
            public void onChangeOpacity(int opacity) {
                super.onChangeOpacity(opacity);
                toolListAdapter.updateItemColorOpacity(toolListAdapter.getCurrentAnnotType(), opacity);

            }
        });
        dialogFragment.setStyleDialogDismissListener(() -> {
            CAnnotStyle style1 = dialogFragment.getAnnotStyle();
            if (style1.getType() == CStyleType.ANNOT_STAMP || style1.getType() == CStyleType.ANNOT_SIGNATURE || style1.getType() == CStyleType.ANNOT_PIC) {
                if (style1.getTextStamp() == null
                        && style1.getStandardStamp() == null
                        && TextUtils.isEmpty(style1.getImagePath())) {
                    pdfView.resetAnnotationType();
                }
            }
        });
        dialogFragment.show(fragmentManager, "annotStyleDialogFragment");
    }

    private void switchAnnotationType(CAnnotToolBean bean) {
        ivSetting.setEnabled(toolListAdapter.annotEnableSetting());
        if (!bean.isSelect()) {
            pdfView.resetAnnotationType();
            pdfView.getCPdfReaderView().getInkDrawHelper().onClean();
            if (annotationChangeListener != null) {
                annotationChangeListener.change(CAnnotationType.UNKNOWN);
            }
            return;
        }
        if (bean.getType() != CAnnotationType.INK) {
            pdfView.getCPdfReaderView().getInkDrawHelper().onClean();
            pdfView.getCPdfReaderView().getInkDrawHelper().setMode(IInkDrawCallback.Mode.DRAW);
        }
        pdfView.getCPdfReaderView().removeAllAnnotFocus();
        switch (bean.getType()) {
            case TEXT:
                pdfView.changeAnnotationType(CPDFAnnotation.Type.TEXT);
                break;
            case INK:
                pdfView.changeAnnotationType(CPDFAnnotation.Type.INK);
                pdfView.getCPdfReaderView().getInkDrawHelper().setEffect(IInkDrawCallback.Effect.NORMAL);
                break;
            case ARROW: {
                pdfView.changeAnnotationType(CPDFAnnotation.Type.LINE);
                CStyleManager styleManager = new CStyleManager(pdfView);
                CAnnotStyle style = styleManager.getStyle(CStyleType.ANNOT_ARROW);
                styleManager.updateStyle(style);
                break;
            }
            case LINE:
                pdfView.changeAnnotationType(CPDFAnnotation.Type.LINE);
                CStyleManager styleManager = new CStyleManager(pdfView);
                CAnnotStyle style = styleManager.getStyle(CStyleType.ANNOT_LINE);
                styleManager.updateStyle(style);
                break;
            case SIGNATURE:
            case STAMP:
            case PIC:
                pdfView.changeAnnotationType(CPDFAnnotation.Type.STAMP);
                showAnnotStyleDialog();
                break;
            default:
                pdfView.changeAnnotationType(CPDFAnnotation.Type.valueOf(bean.getType().name()));
                break;
        }
        if (annotationChangeListener != null) {
            annotationChangeListener.change(bean.getType());
        }
    }

    private Handler handler = new Handler(Looper.getMainLooper());

    private void redoUndoManager() {
        CPDFUndoManager undoManager = pdfView.getCPdfReaderView().getUndoManager();
        ivRedo.setEnabled(undoManager.canRedo());
        ivUndo.setEnabled(undoManager.canRedo());
        pdfView.getCPdfReaderView().getUndoManager().enable(true);
        pdfView.getCPdfReaderView().getUndoManager().addOnUndoHistoryChangeListener((cpdfUndoManager, operation, type) -> {
            boolean canUndo = cpdfUndoManager.canUndo();
            boolean canRedo = cpdfUndoManager.canRedo();
            handler.post(() -> {
                ivUndo.setEnabled(canUndo);
                ivRedo.setEnabled(canRedo);
            });
        });
    }

    private void undo() {
        try {
            if (pdfView != null) {
                pdfView.getCPdfReaderView().getUndoManager().undo();
            }
        } catch (CPDFUndoFailedException e) {
            throw new RuntimeException(e);
        }
    }

    private void redo() {
        try {
            if (pdfView != null) {
                pdfView.getCPdfReaderView().getUndoManager().redo();
            }
        } catch (CPDFRedoFailedException e) {
            throw new RuntimeException(e);
        }
    }

    public void updateItemColor() {
        if (pdfView == null) {
            return;
        }
        CStyleManager styleManager = new CStyleManager(pdfView);
        CAnnotStyle noteStyle = styleManager.getStyle(CStyleType.ANNOT_TEXT);
        toolListAdapter.updateItem(CAnnotationType.TEXT, noteStyle.getColor(), noteStyle.getOpacity());

        CAnnotStyle highStyle = styleManager.getStyle(CStyleType.ANNOT_HIGHLIGHT);
        toolListAdapter.updateItem(CAnnotationType.HIGHLIGHT, highStyle.getColor(), highStyle.getOpacity());

        CAnnotStyle underLineStyle = styleManager.getStyle(CStyleType.ANNOT_UNDERLINE);
        toolListAdapter.updateItem(CAnnotationType.UNDERLINE, underLineStyle.getColor(), underLineStyle.getOpacity());

        CAnnotStyle strikeoutStyle = styleManager.getStyle(CStyleType.ANNOT_STRIKEOUT);
        toolListAdapter.updateItem(CAnnotationType.STRIKEOUT, strikeoutStyle.getColor(), strikeoutStyle.getOpacity());

        CAnnotStyle squigglyStyle = styleManager.getStyle(CStyleType.ANNOT_SQUIGGLY);
        toolListAdapter.updateItem(CAnnotationType.SQUIGGLY, squigglyStyle.getColor(), squigglyStyle.getOpacity());

        CAnnotStyle inkStyle = styleManager.getStyle(CStyleType.ANNOT_INK);
        toolListAdapter.updateItem(CAnnotationType.INK, inkStyle.getColor(), inkStyle.getOpacity());
    }

    public void setAnnotationChangeListener(COnAnnotationChangeListener annotationChangeListener) {
        this.annotationChangeListener = annotationChangeListener;
    }
}
