package com.compdfkit.tools.common.views.pdfproperties.writing;


import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.graphics.Paint.Cap;
import android.graphics.Paint.Join;
import android.graphics.Paint.Style;
import android.graphics.Path;
import android.graphics.RectF;
import android.util.AttributeSet;
import android.view.MotionEvent;
import android.view.View;

import androidx.annotation.Nullable;


public class CKmWritingView extends View {
    private Context mContext;
    protected CKMHandWritingHelper handwritingHelper;
    private Paint erasePathPaint;
    private int erasePathColor = -7829368;

    public CKmWritingView(Context context) {
        super(context);
        this.initView(context);
    }

    public CKmWritingView(Context context, @Nullable AttributeSet attrs) {
        super(context, attrs);
        this.initView(context);
    }

    public CKmWritingView(Context context, @Nullable AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        this.initView(context);
    }

    private void initView(Context context) {
        this.mContext = context;
        this.setWillNotDraw(false);
        this.handwritingHelper = new CKMHandWritingHelper();
        this.erasePathPaint = new Paint();
        this.erasePathPaint.setColor(this.erasePathColor);
        this.erasePathPaint.setStyle(Style.STROKE);
        this.erasePathPaint.setStrokeCap(Cap.ROUND);
        this.erasePathPaint.setStrokeJoin(Join.ROUND);
        this.erasePathPaint.setAntiAlias(true);
        this.erasePathPaint.setStrokeMiter(1.0F);
        this.erasePathPaint.setStrokeWidth(60.0F);
    }

    public void onWindowFocusChanged(boolean hasWindowFocus) {
        super.onWindowFocusChanged(hasWindowFocus);
        if (hasWindowFocus) {
            this.post(() -> {
                if (this.handwritingHelper != null && !this.handwritingHelper.isInited()) {
                    this.handwritingHelper.initCanvas(this.getWidth(), this.getHeight());
                }
            });
        }

    }

    public void setDrawMode(CKMHandWritingHelper.DrawMode drawMode) {
        if (this.handwritingHelper != null) {
            this.handwritingHelper.setDrawMode(drawMode);
        }

    }

    public void setDrawEffect(CKMHandWritingHelper.DrawEffect drawEffect) {
        if (this.handwritingHelper != null) {
            this.handwritingHelper.setDrawEffect(drawEffect);
        }

    }

    public CKMHandWritingHelper.DrawMode getDrawMode() {
        return this.handwritingHelper != null ? this.handwritingHelper.getDrawMode() : CKMHandWritingHelper.DrawMode.DRAW;
    }

    public void setPenWidth(float penWidth) {
        if (this.handwritingHelper != null) {
            this.handwritingHelper.setPenWidth(penWidth);
        }
    }

    public void setEraseWidth(float eraseWidth) {
        if (this.handwritingHelper != null) {
            this.handwritingHelper.setEraseWidth(eraseWidth);
        }
        this.erasePathPaint.setStrokeWidth(eraseWidth);
    }

    public void setColor(int color) {
        if (this.handwritingHelper != null) {
            this.handwritingHelper.setColor(color);
        }
    }

    public void setAlpha(int alpha) {
        if (this.handwritingHelper != null) {
            this.handwritingHelper.setAlpha(alpha);
        }
    }

    public void clean() {
        if (this.handwritingHelper != null) {
            this.handwritingHelper.clean();
        }
    }

    public boolean onTouchEvent(MotionEvent event) {
        if (this.handwritingHelper == null) {
            return false;
        } else {
            if (!this.handwritingHelper.isInited()) {
                this.handwritingHelper.initCanvas(this.getWidth(), this.getHeight());
            }

            switch (event.getActionMasked()) {
                case 0:
                    this.handwritingHelper.onDown(event.getX(), event.getY(), -1.0F, -1.0F);
                    break;
                case 1:
                    if (this.handwritingHelper.onUp(event.getX(), event.getY(), -1.0F, -1.0F)) {
                        this.invalidate();
                    }
                    break;
                case 2:
                    if (this.handwritingHelper.onMove(event.getX(), event.getY(), -1.0F, -1.0F)) {
                        this.invalidate();
                    }
            }

            return true;
        }
    }

    protected void onDraw(Canvas canvas) {
        if (this.handwritingHelper != null) {
            Bitmap bitmap = this.handwritingHelper.getBitmap();
            if (bitmap != null && !bitmap.isRecycled()) {
                canvas.drawBitmap(bitmap, 0.0F, 0.0F, (Paint)null);
            }

            Path erasePath = this.handwritingHelper.getErasePath();
            if (erasePath != null && !erasePath.isEmpty()) {
                canvas.drawPath(erasePath, this.erasePathPaint);
            }
        }

    }

    public boolean isBlank() {
        if (this.handwritingHelper != null) {
            RectF area = this.handwritingHelper.getArea();
            if (area != null && Math.abs(area.width()) > 2.0F && Math.abs(area.height()) > 2.0F) {
                return false;
            }
        }

        return true;
    }

    public Bitmap getBitmap() {
        return this.handwritingHelper == null ? null : this.handwritingHelper.getBitmap();
    }
}
