/**
 * Copyright © 2014-2023 PDF Technologies, Inc. All Rights Reserved.
 * <p>
 * THIS SOURCE CODE AND ANY ACCOMPANYING DOCUMENTATION ARE PROTECTED BY INTERNATIONAL COPYRIGHT LAW
 * AND MAY NOT BE RESOLD OR REDISTRIBUTED. USAGE IS BOUND TO THE ComPDFKit LICENSE AGREEMENT.
 * UNAUTHORIZED REPRODUCTION OR DISTRIBUTION IS SUBJECT TO CIVIL AND CRIMINAL PENALTIES.
 * This notice may not be removed from this file.
 */

package com.compdfkit.tools.common.views.pdfview;

import android.animation.ObjectAnimator;
import android.content.Context;
import android.content.res.TypedArray;
import android.net.Uri;
import android.os.Handler;
import android.os.Looper;
import android.util.AttributeSet;
import android.util.Log;
import android.view.ViewGroup;
import android.widget.LinearLayout;
import android.widget.RelativeLayout;

import androidx.annotation.DrawableRes;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.core.content.ContextCompat;
import androidx.fragment.app.FragmentActivity;
import androidx.interpolator.view.animation.FastOutLinearInInterpolator;

import com.compdfkit.core.annotation.CPDFAnnotation;
import com.compdfkit.core.annotation.form.CPDFWidget;
import com.compdfkit.core.common.CPDFDocumentException;
import com.compdfkit.core.document.CPDFDocument;
import com.compdfkit.core.edit.CPDFEditManager;
import com.compdfkit.tools.R;
import com.compdfkit.tools.common.utils.dialog.CGotoPageDialog;
import com.compdfkit.tools.common.utils.threadpools.CThreadPoolUtils;
import com.compdfkit.tools.common.utils.viewutils.CDimensUtils;
import com.compdfkit.tools.common.utils.viewutils.CViewUtils;
import com.compdfkit.tools.common.views.CVerifyPasswordDialogFragment;
import com.compdfkit.ui.reader.CPDFReaderView;
import com.compdfkit.ui.reader.IReaderViewCallback;
import com.compdfkit.ui.reader.OnFocusedTypeChangedListener;

import java.util.ArrayList;
import java.util.List;


/**
 * compdfkit sdk CPDFReaderView ctrl view <br/>
 * <p>
 * <p>
 * ︳--------------------------------︳<BR/>
 * ︳　　　　　　　    　　 　 |---------︳<BR/>
 * ︳　　　　　　　　　　  　|sliderbar︳<BR/>
 * ︳　　　　　　　　　    　　|---------︳<BR/>
 * ︳　　　　　　　　　　 　　     　　　 ︳<br/>
 * ︳　　　　　　　　　　 　　     　　　 ︳<br/>
 * ︳　　　　　　　　　　 　　     　　　 ︳<br/>
 * ︳　　 　CPDFViewCtrl 　　      　　︳<br/>
 * ︳　　　　　　　　　　 　　     　　　 ︳<br/>
 * ︳　　　　　　　　　　 　　     　　　 ︳<br/>
 * ︳　　　　　　　　　　 　　     　　　 ︳<br/>
 * ︳|---------|　　　　　　 　    　　　 ︳<br/>
 * ︳|indicator|　　　　　　 　    　　　 ︳<br/>
 * ︳|---------|　　　　　　 　    　　　 ︳<br/>
 * ︳--------------------------------︳<br/>
 * <p>
 * STEP 1:<br/>
 * Use CPDFViewCtrl in xml layout <br/>
 * <br/>
 * com.compdfkit.tools.common.views.pdfview.CPDFViewCtrl<br/>
 * android:id="@+id/pdf_reader_view" <br/>
 * android:layout_width="match_parent"<br/>
 * android:layout_height="match_parent"<br/>
 * />
 * <p>
 * <p/>
 * STEP 2:
 * open pdf file <br/>
 * use {@link CPDFViewCtrl#openPDF(String pdfFilePath)}
 * <p>
 * <p/>
 * Congratulations you have completed the pdf file display!!!
 * <br/>
 * <p>
 * <br/>
 * custom attributes:<br/>
 * app:tools_enable_slider_bar="true|false"  <br/>
 * <p/>
 * app:tools_slider_bar_position="left|right"
 * <p/>
 * app:tools_slider_bar_icon="@drawable/xxx"
 * <p/>
 * app:tools_slider_bar_thumbnail_width="120dp"
 * <p/>
 * app:tools_slider_bar_thumbnail_hei="200dp"
 * <p/>
 * app:tools_enable_page_indicator="true|false"
 */
public class CPDFViewCtrl extends RelativeLayout implements IReaderViewCallback, OnFocusedTypeChangedListener {

    private CPDFReaderView cPdfReaderView;

    public CPDFPageIndicatorView indicatorView;

    public CPDFSlideBar slideBar;

    public int currentPageIndex = 0;

    private boolean enableSliderBar = true;

    private CPDFSlideBar.SlideBarPosition slideBarPosition = CPDFSlideBar.SlideBarPosition.RIGHT;

    private int sliderBarThumbnailWidth = 314;

    private int sliderBarThumbnailHeight = 444;

    @DrawableRes
    private int sliderBarIconResId = R.drawable.tools_ic_pdf_slider_bar;

    private boolean enablePageIndicator = true;

    private int pageIndicatorMarginBottom = 0;

    private OnTapMainDocAreaCallback onTapMainDocAreaCallback;

    private boolean isScrolling = false;

    private Handler handler = new Handler(Looper.getMainLooper());

    private ObjectAnimator pageIndicatorAnimator = null;

    private List<OnFocusedTypeChangedListener> pdfViewFocusedListenerList = new ArrayList<>();

    private Runnable hideIndicatorRunnable = () -> {
        if (pageIndicatorAnimator != null) {
            isScrolling = false;
            pageIndicatorAnimator.reverse();
        }
    };

    COnEndScrollCallback endScrollCallback = null;

    public CPDFViewCtrl(@NonNull Context context) {
        this(context, null);
    }

    public CPDFViewCtrl(@NonNull Context context, @Nullable AttributeSet attrs) {
        this(context, attrs, 0);
    }

    public CPDFViewCtrl(@NonNull Context context, @Nullable AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        initAttr(context, attrs);
        initCPDFReaderView();
    }

    private void initAttr(Context context, AttributeSet attributeSet) {
        try {
            TypedArray typedArray = context.obtainStyledAttributes(attributeSet, R.styleable.CPDFReaderView);
            if (typedArray != null) {
                enableSliderBar = typedArray.getBoolean(R.styleable.CPDFReaderView_tools_enable_slider_bar, true);
                int sliderBarPositionEnum = typedArray.getInt(R.styleable.CPDFReaderView_tools_slider_bar_position, 1);
                if (sliderBarPositionEnum == 0) {
                    slideBarPosition = CPDFSlideBar.SlideBarPosition.LEFT;
                } else {
                    slideBarPosition = CPDFSlideBar.SlideBarPosition.RIGHT;
                }
                sliderBarThumbnailWidth = typedArray.getDimensionPixelOffset(R.styleable.CPDFReaderView_tools_slider_bar_thumbnail_width, 314);
                sliderBarThumbnailHeight = typedArray.getDimensionPixelOffset(R.styleable.CPDFReaderView_tools_slider_bar_thumbnail_height, 444);
                sliderBarIconResId = typedArray.getResourceId(R.styleable.CPDFReaderView_tools_slider_bar_icon, R.drawable.tools_ic_pdf_slider_bar);
                enablePageIndicator = typedArray.getBoolean(R.styleable.CPDFReaderView_tools_enable_page_indicator, true);
                pageIndicatorMarginBottom = typedArray.getDimensionPixelOffset(R.styleable.CPDFReaderView_tools_page_indicator_margin_bottom, 0);
                if (enableSliderBar) {
                    slideBar = new CPDFSliderBarView(getContext());
                }
                if (enablePageIndicator) {
                    indicatorView = new CPDFPageIndicatorView(getContext());
                }
                CViewUtils.applyViewBackground(this, ContextCompat.getColor(getContext(), R.color.tools_pdf_view_ctrl_background_color));
                typedArray.recycle();
            }
        } catch (Exception e) {

        }
    }


    private void initCPDFReaderView() {
        cPdfReaderView = new com.compdfkit.ui.reader.CPDFReaderView(getContext());
        cPdfReaderView.setDoublePageMode(false);
        cPdfReaderView.setReaderViewCallback(this);
        cPdfReaderView.setOnFocusedTypeChangedListener(this);
        addView(cPdfReaderView);
    }

    public void resetCPDFReadView() {
        removeAllViews();
        initCPDFReaderView();
    }

    public void setAllowAddAndEditAnnot(boolean allow) {
        cPdfReaderView.setAllowAddAndEditAnnot(allow);
    }

    /**
     * This method opens a PDF file located at the given file path.
     *
     * @param pdfFilePath The file path of the PDF file to be opened.
     */
    public void openPDF(String pdfFilePath) {
        // Create a new instance of the CPDFDocument class, passing in the current context.
        CPDFDocument cpdfDocument = new CPDFDocument(getContext());
        // Attempt to open the PDF file at the given file path using the open method of the CPDFDocument class.
        CPDFDocument.PDFDocumentError pdfDocumentError = cpdfDocument.open(pdfFilePath);
        setPDFDocument(cpdfDocument, pdfFilePath, pdfDocumentError, null);
    }

    public void openPDF(Uri pdfUri) {
        // Create a new instance of the CPDFDocument class, passing in the current context.
        CPDFDocument cpdfDocument = new CPDFDocument(getContext());
        // Attempt to open the PDF file at the given file path using the open method of the CPDFDocument class.
        CPDFDocument.PDFDocumentError pdfDocumentError = cpdfDocument.open(pdfUri);
        setPDFDocument(cpdfDocument, pdfUri, pdfDocumentError, null);
    }

    public void openPDF(Uri pdfUri, COnOpenPdfFinishCallback openPdfFinishCallback) {
        // Create a new instance of the CPDFDocument class, passing in the current context.
        CPDFDocument cpdfDocument = new CPDFDocument(getContext());
        // Attempt to open the PDF file at the given file path using the open method of the CPDFDocument class.
        CPDFDocument.PDFDocumentError pdfDocumentError = cpdfDocument.open(pdfUri);
        setPDFDocument(cpdfDocument, pdfUri, pdfDocumentError, openPdfFinishCallback);
    }

    private void setPDFDocument(CPDFDocument cpdfDocument, Object pdf, CPDFDocument.PDFDocumentError error, COnOpenPdfFinishCallback openPdfFinishCallback) {
        // Switch on the result of the open method to handle the different possible outcomes.
        switch (error) {
            // If the PDF file was successfully opened, set the PDF document of the CPdfReaderView instance to the opened document.
            case PDFDocumentErrorSuccess:
                cPdfReaderView.setPDFDocument(cpdfDocument);
                cPdfReaderView.setDisplayPageIndex(0);
                initCPDFSliderBar();
                addPageIndicator();
                if (cPdfReaderView.getEditManager() != null) {
                    cPdfReaderView.getEditManager().disable();
                }
                if (openPdfFinishCallback != null) {
                    openPdfFinishCallback.onOpenPdfFinishCallback();
                }
                break;
            // If the PDF file requires a password to be opened, do nothing for now.
            case PDFDocumentErrorPassword:
                CVerifyPasswordDialogFragment verifyPasswordDialogFragment;
                if (pdf instanceof String) {
                    verifyPasswordDialogFragment = CVerifyPasswordDialogFragment.newInstance(cpdfDocument, (String) pdf);
                } else {
                    verifyPasswordDialogFragment = CVerifyPasswordDialogFragment.newInstance(cpdfDocument, (Uri) pdf);
                }
                verifyPasswordDialogFragment.setVerifyCompleteListener(document -> {
                    cPdfReaderView.setPDFDocument(document);
                    cPdfReaderView.setDisplayPageIndex(0);
                    initCPDFSliderBar();
                    addPageIndicator();
                    if (cPdfReaderView.getEditManager() != null) {
                        cPdfReaderView.getEditManager().disable();
                    }
                    if (openPdfFinishCallback != null) {
                        openPdfFinishCallback.onOpenPdfFinishCallback();
                    }
                });
                if (getContext() instanceof FragmentActivity){
                    verifyPasswordDialogFragment.show(((FragmentActivity) getContext()).getSupportFragmentManager(), "verifyPwdDialog");
                }
                break;
            // For all other errors, do nothing for now.
            default:
                break;
        }
    }

    public void savePDF(COnSaveCallback callback) {
        CThreadPoolUtils.getInstance().execute(() -> {
            cPdfReaderView.pauseAllRenderProcess();
            CPDFDocument document = cPdfReaderView.getPDFDocument();
            if (document.hasChanges()) {
                try {
                    boolean success = document.save();
                    if (!success) {
                        document.save(CPDFDocument.PDFDocumentSaveType.PDFDocumentSaveNoIncremental);
                    }
                    if (callback != null) {
                        callback.callback(document.getAbsolutePath(), document.getUri());
                    }
                } catch (CPDFDocumentException e) {
                    e.printStackTrace();
                }
            } else {
                if (callback != null) {
                    callback.callback(document.getAbsolutePath(), document.getUri());
                }
            }
        });
    }

    public void changeFormType(CPDFWidget.WidgetType widgetType){
        cPdfReaderView.setCurrentFocusedType(CPDFAnnotation.Type.WIDGET);
        cPdfReaderView.setCurrentFocusedFormType(widgetType);
        cPdfReaderView.setTouchMode(CPDFReaderView.TouchMode.ADD_ANNOT);
    }

    public void resetFormType(){
        cPdfReaderView.setTouchMode(CPDFReaderView.TouchMode.BROWSE);
        cPdfReaderView.setCurrentFocusedType(CPDFAnnotation.Type.WIDGET);
        cPdfReaderView.setCurrentFocusedFormType(CPDFWidget.WidgetType.Widget_Unknown);
    }

    public void changeAnnotationType(CPDFAnnotation.Type type) {
        cPdfReaderView.setTouchMode(CPDFReaderView.TouchMode.ADD_ANNOT);
        cPdfReaderView.setCurrentFocusedType(type);
    }

    public void resetAnnotationType() {
        cPdfReaderView.setTouchMode(CPDFReaderView.TouchMode.BROWSE);
        cPdfReaderView.setCurrentFocusedType(CPDFAnnotation.Type.UNKNOWN);
    }

    public void exitEditMode(){
        CPDFEditManager editManager = cPdfReaderView.getEditManager();
        if (editManager != null && editManager.isEditMode()) {
            editManager.endEdit();
        }
    }

    public com.compdfkit.ui.reader.CPDFReaderView getCPdfReaderView() {
        return cPdfReaderView;
    }

    public void setEndScrollCallback(COnEndScrollCallback callback) {
        this.endScrollCallback = callback;
    }

    @Override
    public void onTypeChanged(CPDFAnnotation.Type type) {
        if (pdfViewFocusedListenerList != null) {
            for (OnFocusedTypeChangedListener onFocusedTypeChangedListener : pdfViewFocusedListenerList) {
                onFocusedTypeChangedListener.onTypeChanged(type);
            }
        }
    }

    @Override
    public void onTapMainDocArea() {
        if (onTapMainDocAreaCallback != null) {
            onTapMainDocAreaCallback.onTapMainDocArea();
        }
    }

    @Override
    public void onMoveToChild(int pageIndex) {
        currentPageIndex = pageIndex;
        if (slideBar != null) {
            slideBar.setPageIndexByAnimation(pageIndex, 500);
        }
        if (indicatorView != null) {
            indicatorView.setCurrentPageIndex(pageIndex);
        }
    }

    @Override
    public void onEndScroll() {
        hidePageIndicator();
        if (endScrollCallback != null) {
            endScrollCallback.onEndScrollCallback();
        }
    }

    @Override
    public void onScrolling() {
        isHiding = false;
        handler.removeCallbacks(hideIndicatorRunnable);
        if (!isScrolling) {
            isScrolling = true;
            showPageIndicator();
        }
    }

    @Override
    public void onRecordLastJumpPageNum(int i) {

    }

    private void initCPDFSliderBar() {
        if (!enableSliderBar) {
            return;
        }
        CPDFSliderBarView cpdfSliderBarView = (CPDFSliderBarView) slideBar;
        cpdfSliderBarView.initWithPDFView(this);
        cpdfSliderBarView.initSliderBar(CPDFSlideBar.SlideBarPosition.RIGHT, sliderBarThumbnailWidth, sliderBarThumbnailHeight);
        cpdfSliderBarView.setSlideBarBitmap(sliderBarIconResId);
        setSliderBar(cpdfSliderBarView);
    }

    private void setSliderBar(CPDFSlideBar cpdfSlideBar) {
        enableSliderBar = true;
        if (slideBar.getParent() != null) {
            ViewGroup viewGroup = (ViewGroup) slideBar.getParent();
            viewGroup.removeAllViews();
        }
        slideBar = cpdfSlideBar;
        LinearLayout linearLayout = new LinearLayout(getContext());
        LayoutParams layoutParams = new LayoutParams(LayoutParams.WRAP_CONTENT, LayoutParams.MATCH_PARENT);
        linearLayout.setLayoutParams(layoutParams);
        int actionBarHeight = CViewUtils.getActionBarSize(getContext());
        if (actionBarHeight > 0) {
            layoutParams.setMargins(0, actionBarHeight, 0, actionBarHeight);
        }
        linearLayout.addView(slideBar);
        slideBar.setLayoutParams(new LinearLayout.LayoutParams(LayoutParams.WRAP_CONTENT, LayoutParams.MATCH_PARENT));
        addView(linearLayout);
    }

    /**
     * Add a page indicator in the lower left corner of the page.
     * After clicking, a page number jump input box will pop up, and you can jump to the corresponding PDF page
     * page input limit: 1 ~ pdf total page
     *
     * @see CPDFPageIndicatorView
     */
    private void addPageIndicator() {
        if (!enablePageIndicator) {
            return;
        }
        removeView(indicatorView);
        RelativeLayout.LayoutParams layoutParams = new RelativeLayout.LayoutParams(LayoutParams.WRAP_CONTENT, LayoutParams.WRAP_CONTENT);
        layoutParams.addRule(RelativeLayout.ALIGN_PARENT_BOTTOM);
        layoutParams.addRule(RelativeLayout.CENTER_HORIZONTAL);
        int margin = CDimensUtils.dp2px(getContext(), 16);
        layoutParams.setMargins(margin, 0, 0, pageIndicatorMarginBottom);
        indicatorView.setLayoutParams(layoutParams);
        indicatorView.setAlpha(0F);
        addView(indicatorView);

        int totalPageCount = cPdfReaderView.getPDFDocument().getPageCount();
        indicatorView.setTotalPage(totalPageCount);
        indicatorView.setCurrentPageIndex(0);
        indicatorView.setPageIndicatorClickListener(pageIndex -> {
            CGotoPageDialog dialog = new CGotoPageDialog(getContext());
            dialog.setHintInputText(getContext().getString(R.string.tools_page) + String.format(" (%d/%d)", 1, totalPageCount));
            dialog.setPageCount(totalPageCount);
            dialog.setOnPDFDisplayPageIndexListener(page -> {
                if (page <= totalPageCount && page > 0) {
                    cPdfReaderView.setDisplayPageIndex(page - 1, true);
                }
            });
            dialog.show();
        });
        pageIndicatorAnimator = ObjectAnimator.ofFloat(indicatorView, "alpha", 0F, 1F);
        pageIndicatorAnimator.setDuration(100);
        pageIndicatorAnimator.setInterpolator(new FastOutLinearInInterpolator());
        showPageIndicator();
        hidePageIndicator();
    }

    private boolean isHiding = false;

    private void hidePageIndicator() {
        if (!isHiding){
            isHiding = true;
            handler.postDelayed(hideIndicatorRunnable, 3000);
        }
    }

    private void showPageIndicator() {
        if (pageIndicatorAnimator != null && indicatorView.getAlpha() != 1.0F) {
            Log.e("页码","显示页码");
            isHiding = false;
            pageIndicatorAnimator.start();
        }
    }

    public void addOnPDFFocusedTypeChangeListener(OnFocusedTypeChangedListener listener) {
        pdfViewFocusedListenerList.add(listener);
    }

    public void setOnTapMainDocAreaCallback(OnTapMainDocAreaCallback onTapMainDocAreaCallback) {
        this.onTapMainDocAreaCallback = onTapMainDocAreaCallback;
    }


    public interface OnTapMainDocAreaCallback {
        void onTapMainDocArea();
    }

    public interface COnSaveCallback {
        void callback(String filePath, Uri pdfUri);
    }

    public interface COnEndScrollCallback {
        void onEndScrollCallback();
    }

    public interface COnOpenPdfFinishCallback {
        void onOpenPdfFinishCallback();
    }
}
