/**
 * Copyright © 2014-2023 PDF Technologies, Inc. All Rights Reserved.
 * <p>
 * THIS SOURCE CODE AND ANY ACCOMPANYING DOCUMENTATION ARE PROTECTED BY INTERNATIONAL COPYRIGHT LAW
 * AND MAY NOT BE RESOLD OR REDISTRIBUTED. USAGE IS BOUND TO THE ComPDFKit LICENSE AGREEMENT.
 * UNAUTHORIZED REPRODUCTION OR DISTRIBUTION IS SUBJECT TO CIVIL AND CRIMINAL PENALTIES.
 * This notice may not be removed from this file.
 */

package com.compdfkit.tools.viewer.pdfoutline.adapter;

import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import androidx.annotation.NonNull;
import androidx.appcompat.widget.AppCompatImageView;
import androidx.appcompat.widget.AppCompatTextView;
import androidx.constraintlayout.widget.ConstraintLayout;
import androidx.core.content.ContextCompat;
import androidx.recyclerview.widget.RecyclerView;

import com.compdfkit.tools.R;
import com.compdfkit.tools.common.interfaces.COnSetPDFDisplayPageIndexListener;
import com.compdfkit.tools.common.views.pdfview.CPDFViewCtrl;
import com.compdfkit.tools.viewer.pdfoutline.bean.COutlineData;
import com.compdfkit.tools.viewer.pdfoutline.data.COutlineDatas;
import com.compdfkit.ui.reader.CPDFReaderView;

import java.util.ArrayList;
import java.util.List;


/**
 * PDF outline data list. When the outline has sub-levels,<br/>
 * the sub-level outline list can be expanded or collapsed by clicking.<br/>
 * Clicking on a non-collapsible outline will jump to the corresponding PDF page.<br/>
 * <p>
 * <p/>
 * If you do not like this list style and want to obtain PDF outline data for display on your own,<br/>
 * you can get the PDF outline data by calling "cpdfView.getCPdfReaderView().getPDFDocument().getOutlineRoot()". <br/>
 * Alternatively, you can use the preprocessed data that we have provided for you.<br/>
 * {@link COutlineDatas#getOutlineList(CPDFViewCtrl cpdfView, int maxLevel)}
 *
 * @see CPDFViewCtrl
 * @see CPDFReaderView#getPDFDocument()
 * @see COutlineDatas
 */
public class COutlineListAdapter extends RecyclerView.Adapter<COutlineListAdapter.OutlineItemViewHolder> {

    private static final String REFRESH_ARROW = "refresh_arrow";

    private ArrayList<COutlineData> list = new ArrayList<>();

    private COnSetPDFDisplayPageIndexListener outlineClickListener;

    /**
     * set outline list data
     *
     * @param data new outline list data
     */
    public void setList(ArrayList<COutlineData> data) {
        list.clear();
        list.addAll(data);
    }

    @NonNull
    @Override
    public OutlineItemViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        return new OutlineItemViewHolder(LayoutInflater.from(parent.getContext()).inflate(R.layout.tools_bota_outline_list_item, parent, false));
    }

    @Override
    public void onBindViewHolder(@NonNull OutlineItemViewHolder holder, int position) {
        COutlineData item = list.get(holder.getAdapterPosition());
        holder.tvItemTitle.setText(item.getTitle());
        if (item.getPageIndex() < 0) {
            holder.tvItemPageIndex.setText("");
        } else {
            holder.tvItemPageIndex.setText(String.valueOf(item.getPageIndex() + 1));
        }
        ConstraintLayout.LayoutParams layoutParams = (ConstraintLayout.LayoutParams) holder.ivItemArrow.getLayoutParams();
        layoutParams.setMarginStart(item.getLevelMargin(holder.itemView.getContext()));
        holder.ivItemArrow.setLayoutParams(layoutParams);
        holder.ivItemArrow.setVisibility(item.childOutlineIsEmpty() ? View.INVISIBLE : View.VISIBLE);
        holder.ivItemArrow.setImageResource(item.isExpand() ? R.drawable.tools_ic_arrow_down : R.drawable.tools_ic_right);
        holder.clRoot.setBackgroundColor(ContextCompat.getColor(holder.itemView.getContext(), item.getLevel() == 1 ? R.color.tools_reader_setting_head_bg_color : R.color.tools_color_background));
        holder.itemView.setOnClickListener(v -> {
            // Check if the child outline of the item is empty
            // If the child outline is empty and an outlineClickListener is set, display the page
            if (outlineClickListener != null) {
                outlineClickListener.displayPage(item.getPageIndex());
            }
        });
        holder.ivItemArrow.setOnClickListener(v -> {
            // If the child outline is not empty, check if the item is already expanded
            if (item.isExpand()) {
                // If the item is already expanded, collapse it
                collapseItem(holder.getAdapterPosition(), item);
            } else {
                // If the item is not expanded, expand it
                expandItem(holder.getAdapterPosition(), item);
            }
        });
    }

    @Override
    public void onBindViewHolder(@NonNull OutlineItemViewHolder holder, int position, @NonNull List<Object> payloads) {
        if (payloads.isEmpty()) {
            super.onBindViewHolder(holder, position, payloads);
        } else {
            for (Object payload : payloads) {
                if (payload == REFRESH_ARROW) {
                    COutlineData item = list.get(position);
                    holder.ivItemArrow.setImageResource(item.isExpand() ? R.drawable.tools_ic_arrow_down : R.drawable.tools_ic_right);
                }
            }
        }
    }

    /**
     * Expand the given item and add its child items to the list.
     *
     * @param positionStart the starting position of the given item in the list
     * @param item          the item to be expanded
     */
    private void expandItem(int positionStart, COutlineData item) {
        // Set expand flag of the item to true
        item.setExpand(true);
        // Add child items of the expanded item to the list
        list.addAll(positionStart + 1, item.getChildOutline());
        // Notify the adapter that the arrow status of the clicked item has changed
        notifyItemChanged(positionStart, REFRESH_ARROW);
        // Notify the adapter that data has been inserted
        notifyItemRangeInserted(positionStart + 1, item.getChildOutline().size());
    }

    /**
     * Collapse the given item and its child items.
     *
     * @param positionStart The position of the item in the adapter.
     * @param item          The item to collapse.
     */
    private void collapseItem(int positionStart, COutlineData item) {
        int removeCount = loopCollapseItem(item);
        notifyItemChanged(positionStart, REFRESH_ARROW);
        notifyItemRangeRemoved(positionStart + 1, removeCount);
    }

    /**
     * Recursively collapse the given item and its child items.
     *
     * @param outlineData The item to collapse.
     * @return The number of items removed from the adapter.
     */
    private int loopCollapseItem(COutlineData outlineData) {
        outlineData.setExpand(false);
        int removeCount = 0;
        for (COutlineData cOutlineData : outlineData.getChildOutline()) {
            if (!cOutlineData.childOutlineIsEmpty()) {
                removeCount += loopCollapseItem(cOutlineData);
            }
            boolean success = list.remove(cOutlineData);
            if (success) {
                removeCount++;
            }
        }
        return removeCount;
    }

    @Override
    public int getItemCount() {
        return list.size();
    }

    static class OutlineItemViewHolder extends RecyclerView.ViewHolder {
        AppCompatImageView ivItemArrow;

        AppCompatTextView tvItemTitle;

        AppCompatTextView tvItemPageIndex;

        ConstraintLayout clRoot;

        public OutlineItemViewHolder(@NonNull View itemView) {
            super(itemView);
            clRoot = itemView.findViewById(R.id.cl_root);
            ivItemArrow = itemView.findViewById(R.id.iv_outline_item_arrow);
            tvItemTitle = itemView.findViewById(R.id.tv_outline_item_title);
            tvItemPageIndex = itemView.findViewById(R.id.tv_outline_item_page);
        }
    }

    public void setOutlineClickListener(COnSetPDFDisplayPageIndexListener outlineClickListener) {
        this.outlineClickListener = outlineClickListener;
    }
}
